package cc.fascinated.bat.features.autorole;

import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.autorole.profile.AutoRoleProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.FeatureService;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.events.guild.member.GuildMemberJoinEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@Log4j2(topic = "AutoRole Listener")
public class AutoRoleListener implements EventListener {
    private final FeatureService featureService;

    @Autowired
    public AutoRoleListener(@NonNull FeatureService featureService) {
        this.featureService = featureService;
    }

    @Override
    public void onGuildMemberJoin(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull GuildMemberJoinEvent event) {
        AutoRoleFeature autoRoleFeature = featureService.getFeature(AutoRoleFeature.class);
        if (!guild.getFeatureProfile().isFeatureEnabled(autoRoleFeature)) { // Check if the feature is enabled
            return;
        }

        AutoRoleProfile profile = guild.getProfile(AutoRoleProfile.class);
        if (profile.getRoles().isEmpty()) {
            return;
        }

        List<String> toRemove = new ArrayList<>();
        for (Role role : profile.getRoles()) {
            if (event.getGuild().getRoleById(role.getId()) == null) {
                toRemove.add(role.getId());
                continue;
            }
            event.getGuild().addRoleToMember(event.getMember(), role).queue();
        }
        toRemove.forEach(profile::removeRole);
        log.info("Gave user \"{}\" {} auto roles in guild \"{}\"{}",
                user.getId(),
                profile.getRoles().size(),
                guild.getName(),
                toRemove.isEmpty() ? "" : " and removed %s invalid roles from the profile".formatted(toRemove.size())
        );
    }
}
