package cc.fascinated.bat.features.base.commands.general.banner;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.Category;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.User;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import net.dv8tion.jda.api.utils.ImageProxy;
import org.springframework.stereotype.Component;

/**
 * @author Nick (okNick)
 */
@Component("banner:user.sub")
@CommandInfo(name = "user", description = "View the banner of a user", guildOnly = false, category = Category.GENERAL)
public class UserSubCommand extends BatCommand {
    public UserSubCommand() {
        super.addOptions(new OptionData(OptionType.USER, "user", "The user to view the banner of", true));
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        OptionMapping userOption = event.getOption("user");
        if (userOption == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("You must provide a user to view the banner of!")
                            .build())
                    .queue();
            return;
        }

        User target = userOption.getAsUser();
        ImageProxy banner = target.retrieveProfile().complete().getBanner();
        if (banner == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("**%s** does not have a banner!".formatted(target.getName()))
                            .build())
                    .queue();
            return;
        }

        event.replyEmbeds(EmbedUtils.genericEmbed()
                .setAuthor("%s's Banner".formatted(target.getName()))
                .setImage(banner.getUrl(512))
                .build()
        ).queue();
    }
}