package cc.fascinated.bat.features.counter;

import cc.fascinated.bat.common.Serializable;
import com.google.gson.Gson;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import org.bson.Document;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@Component
public class CounterProfile extends Serializable {
    /**
     * The counter channels in this guild
     */
    private final List<CounterChannel> counters = new ArrayList<>();

    /**
     * Gets the counter channel by the channel id
     *
     * @param channelId the id of the channel
     * @return the counter channel
     */
    public CounterChannel getCounterChannel(String channelId) {
        for (CounterChannel counter : counters) {
            if (counter.getChannelId().equals(channelId)) {
                return counter;
            }
        }
        return null;
    }

    /**
     * Removes the channel from the counter channels
     *
     * @param channel the channel to remove
     */
    public void removeChannel(TextChannel channel) {
        CounterChannel counterChannel = getCounterChannel(channel.getId());
        if (counterChannel != null) {
            counters.remove(counterChannel);
        }
    }

    /**
     * Sets up the channel as a counter channel
     *
     * @param channel the channel to set as a counter channel
     * @param breakable whether the counter should reset when a user inputs the wrong number or a non number
     */
    public void setupChannel(TextChannel channel, boolean breakable) {
        counters.add(new CounterChannel(channel.getId(), 0, breakable));
    }

    @Override
    public void load(Document document, Gson gson) {
        for (Document counterDocument : document.getList("counters", Document.class, new ArrayList<>())) {
            counters.add(new CounterChannel(
                    counterDocument.getString("channelId"),
                    counterDocument.getInteger("currentCount"),
                    counterDocument.getBoolean("breakable")
            ));
        }
    }

    @Override
    public Document serialize(Gson gson) {
        Document document = new Document();
        List<Document> counterDocuments = new ArrayList<>();
        for (CounterChannel counter : counters) {
            Document counterDocument = new Document();
            counterDocument.append("channelId", counter.getChannelId());
            counterDocument.append("currentCount", counter.getCurrentCount());
            counterDocument.append("breakable", counter.isBreakable());
            counterDocuments.add(counterDocument);
        }
        document.append("counters", counterDocuments);
        return document;
    }

    @Override
    public void reset() {
        counters.clear();
    }
}
